@@ -3,8 +3,9 @@ import json
 import logging
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
+from itertools import chain
 from pathlib import Path
-from typing import Dict, List, Optional, Type, TypeVar
+from typing import Dict, Iterator, List, Mapping, Optional, Type, TypeVar
 
 import numpy as np
 
@@ -204,7 +205,54 @@ class CovarianceBasedScaleBalancer(ScaleBalancerBase):
         return vec_out
 
 
-class EncodingRule(Dict[Type[ElementBase], EncoderBase]):
+class EncodingRuleBase(Mapping[Type[ElementBase], EncoderBase]):
+    @abstractmethod
+    def apply(self, elem_dict: ElementDict) -> np.ndarray:
+        pass
+
+    @abstractmethod
+    def inverse_apply(self, vector_processed: np.ndarray) -> ElementDict:
+        pass
+
+    @property
+    @abstractmethod
+    def dimension_list(self) -> List[int]:
+        """get dimension list for each encoders output"""
+
+    @property
+    def type_bound_table(self) -> Dict[Type[ElementBase], slice]:
+        bounds = get_bound_list(self.dimension_list)
+
+        table = {}
+        for encoder, bound in zip(self.values(), bounds):
+            table[encoder.elem_type] = bound
+        return table
+
+    def apply_to_episode_data(self, episode_data: EpisodeData) -> np.ndarray:
+        vec_list = [self.apply(episode_data[i]) for i in range(len(episode_data))]
+        return np.array(vec_list)
+
+    def apply_to_episode_bundle(self, bundle: EpisodeBundle) -> List[np.ndarray]:
+        def elem_types_to_primitive_elem_set(elem_type_list: List[Type[ElementBase]]):
+            primitve_elem_type_list = []
+            for elem_type in elem_type_list:
+                if issubclass(elem_type, PrimitiveElementBase):
+                    primitve_elem_type_list.append(elem_type)
+                elif issubclass(elem_type, CompositeImageBase):
+                    primitve_elem_type_list.extend(elem_type.image_types)
+            return set(primitve_elem_type_list)
+
+        bundle_elem_types = elem_types_to_primitive_elem_set(list(bundle.types()))
+        required_elem_types = elem_types_to_primitive_elem_set(list(self.keys()))
+        assert required_elem_types <= bundle_elem_types
+
+        vector_seq_list = [self.apply_to_episode_data(data) for data in bundle]
+
+        assert vector_seq_list[0].ndim == 2
+        return vector_seq_list
+
+
+class EncodingRule(Dict[Type[ElementBase], EncoderBase], EncodingRuleBase):
     scale_balancer: ScaleBalancerBase
 
     def apply(self, elem_dict: ElementDict) -> np.ndarray:
@@ -233,53 +281,17 @@ class EncodingRule(Dict[Type[ElementBase], EncoderBase]):
             elem_dict[elem_type] = encoder.backward(vec)
         return elem_dict
 
-    def apply_to_episode_data(self, episode_data: EpisodeData) -> np.ndarray:
-        def encode_and_postprocess(elem_type, encoder) -> np.ndarray:
-            sequence = episode_data.get_sequence_by_type(elem_type)
-            vectors = [encoder.forward(e) for e in sequence]
-            return np.stack(vectors)
-
-        vector_seq = np.hstack([encode_and_postprocess(k, v) for k, v in self.items()])
-        vector_seq_processed = np.array([self.scale_balancer.apply(e) for e in vector_seq])
-        assert_equal_with_message(vector_seq_processed.ndim, 2, "vector_seq dim")
-        return vector_seq_processed
-
-    def apply_to_episode_bundle(self, bundle: EpisodeBundle) -> List[np.ndarray]:
-        def elem_types_to_primitive_elem_set(elem_type_list: List[Type[ElementBase]]):
-            primitve_elem_type_list = []
-            for elem_type in elem_type_list:
-                if issubclass(elem_type, PrimitiveElementBase):
-                    primitve_elem_type_list.append(elem_type)
-                elif issubclass(elem_type, CompositeImageBase):
-                    primitve_elem_type_list.extend(elem_type.image_types)
-            return set(primitve_elem_type_list)
-
-        bundle_elem_types = elem_types_to_primitive_elem_set(list(bundle.types()))
-        required_elem_types = elem_types_to_primitive_elem_set(list(self.keys()))
-        assert required_elem_types <= bundle_elem_types
-
-        vector_seq_list = [self.apply_to_episode_data(data) for data in bundle]
-
-        assert vector_seq_list[0].ndim == 2
-        return vector_seq_list
-
     @property
     def dimension(self) -> int:
         return sum(encoder.output_size for encoder in self.values())
 
     @property
-    def encode_order(self) -> List[Type[ElementBase]]:
-        return list(self.keys())
+    def dimension_list(self) -> List[int]:
+        return [encoder.output_size for encoder in self.values()]
 
     @property
-    def type_bound_table(self) -> Dict[Type[ElementBase], slice]:
-        dims = [encoder.output_size for encoder in self.values()]
-        bounds = get_bound_list(dims)
-
-        table = {}
-        for encoder, bound in zip(self.values(), bounds):
-            table[encoder.elem_type] = bound
-        return table
+    def encode_order(self) -> List[Type[ElementBase]]:
+        return list(self.keys())
 
     def __str__(self) -> str:
         string = "total dim: {}".format(self.dimension)
@@ -325,3 +337,49 @@ class EncodingRule(Dict[Type[ElementBase], EncoderBase]):
                 vector_seq_concated, dims
             )
         return rule
+
+
+@dataclass
+class CompositeEncodingRule(EncodingRuleBase):
+    rules: List[EncodingRule]
+
+    def __post_init__(self):
+        key_set_list = [set(rule.keys()) for rule in self.rules]
+        set_entire = set.union(*key_set_list)
+        no_intersection = len(set_entire) == sum([len(rule.keys()) for rule in self.rules])
+        assert no_intersection
+
+    def apply(self, edict: ElementDict) -> np.ndarray:
+        vec = np.hstack([rule.apply(edict) for rule in self.rules])
+        return vec
+
+    def inverse_apply(self, vector: np.ndarray) -> ElementDict:
+        head = 0
+        elems = []
+        for rule in self.rules:
+            tail = head + rule.dimension
+            edict = rule.inverse_apply(vector[head:tail])
+            elems.extend(list(edict.values()))
+            head = tail
+        edict_merged = ElementDict(elems)
+        return edict_merged
+
+    @property
+    def dimension_list(self) -> List[int]:
+        dims = []
+        for rule in self.rules:
+            for encoder in rule.values():
+                dims.append(encoder.output_size)
+        return dims
+
+    def __getitem__(self, key: Type[ElementBase]) -> EncoderBase:
+        for rule in self.rules:
+            if key in rule:
+                return rule[key]
+        raise KeyError
+
+    def __iter__(self) -> Iterator[Type[ElementBase]]:
+        return chain(*[rule.__iter__() for rule in self.rules])
+
+    def __len__(self) -> int:
+        return sum([len(rule) for rule in self.rules])
